from abc import ABC, abstractmethod
from typing import Callable, Dict


class ViewInteractor(ABC):
    @abstractmethod
    def set_callbacks(self, callbacks: Dict[str, Callable]):
        raise NotImplementedError

    @abstractmethod
    def display_error(self):
        raise NotImplementedError

    @abstractmethod
    def execute(self, *args, **kwargs):
        raise NotImplementedError



import json
from datetime import datetime
from typing import Dict, List
from uuid import uuid4

import numpy as np

from chosco.annotation.domain.aggregate_roots.media_item import MediaItem


class AnnotationSession:
    @staticmethod
    def create(
        media_items: List[MediaItem],
        users: List[str],
        number_of_annotations_per_media: int,
    ):
        for media_item in media_items:
            assert isinstance(
                media_item, MediaItem
            ), "Error in AnnotationSession. media_items must be a list of MediaItems"

        created_at = datetime.utcnow().strftime("%B %d %Y - %H:%M:%S")
        users = users
        number_of_media_items = len(media_items)
        number_of_users = len(users)
        number_of_annotations_per_media = number_of_annotations_per_media

        media_items_indexes = np.repeat(
            np.arange(number_of_media_items), number_of_annotations_per_media
        )
        tasks_list = [[] for i in range(number_of_users)]
        for i, idx in enumerate(media_items_indexes):
            row_id = i % number_of_users
            tasks_list[row_id].append(media_items[idx])

        tasks = {}
        for i, user in enumerate(users):
            tasks[user] = tasks_list[i]

        return AnnotationSession(
            str(uuid4()),
            created_at,
            number_of_media_items,
            number_of_users,
            number_of_annotations_per_media,
            tasks,
        )

    def __init__(
        self,
        id: str,
        created_at: str,
        number_of_ids: int,
        number_of_users: int,
        number_of_annotations_per_media: int,
        tasks: Dict[str, List[MediaItem]],
    ):
        self.id = id
        self.created_at = created_at
        self.number_of_ids = number_of_ids
        self.number_of_users = number_of_users
        self.number_of_annotations_per_media = number_of_annotations_per_media
        self.tasks = tasks

    def to_dict(self):
        tasks_dict = {}
        for username, tasks in self.tasks.items():
            tasks_dict[username] = []
            for media_item in tasks:
                tasks_dict[username].append(media_item.to_dict())

        return {
            "id": self.id,
            "created_at": self.created_at,
            "number_of_ids": self.number_of_ids,
            "number_of_users": self.number_of_users,
            "number_of_annotations_per_media": self.number_of_annotations_per_media,
            "tasks": tasks_dict,
        }

    def __repr__(self):
        return json.dumps(self.to_dict(), indent=4)

    def __eq__(self, other):
        return self.to_dict() == other.to_dict()

    def save(self, filename):
        with open(filename, "w") as outfile:
            json.dump(self.to_dict(), outfile, indent=4)

    @staticmethod
    def from_filename(filename):
        with open(filename, "r") as infile:
            kdict = json.load(infile)
            tasks = kdict.get("tasks")
            if tasks:
                tasks_dict = {}
                for username, tasks_list in tasks.items():
                    tasks_dict[username] = []
                    for media_item in tasks_list:
                        tasks_dict[username].append(MediaItem.from_dict(media_item))
            else:
                tasks_dict = tasks
            return AnnotationSession(
                kdict.get("id"),
                kdict.get("created_at"),
                kdict.get("number_of_ids"),
                kdict.get("number_of_users"),
                kdict.get("number_of_annotations_per_media"),
                tasks_dict,
            )

    def get_tasks_by_user(self, username: str) -> List[MediaItem]:
        return self.tasks.get(username)



from chosco.annotation.domain.aggregate_roots.media_item import MediaItem


class AnnotationTaskStatus:
    def __init__(
        self,
        index: int,
        number_of_annotations: int,
        total_annotations: int,
        current_task: MediaItem = None,
    ):
        self.index = index
        self.number_of_annotations = number_of_annotations
        self.total_annotations = total_annotations
        self.current_task = current_task

    def get_progress(self):
        return f"{self.number_of_annotations+1} of {self.total_annotations}"



from typing import Tuple

from IPython.core.display import Image

from chosco.annotation.domain.aggregate_roots.media_item import MediaItem
from chosco.annotation.domain.annotation_task_provider import AnnotationTaskProvider
from chosco.annotation.domain.annotation_task_status import AnnotationTaskStatus
from chosco.media.repository.domain.media_repository import MediaRepository


class MediaRetriever:
    def __init__(
        self,
        annotation_task_provider: AnnotationTaskProvider,
        repository: MediaRepository,
        image_width: int = 1200,
        image_height: int = None,
    ):
        self.annotation_task_provider = annotation_task_provider
        self.repository = repository
        self.image_width = image_width
        self.image_height = image_height
        self.media_item = None

    def previous_media(self) -> Tuple[MediaItem, Image]:
        self.media_item = self.annotation_task_provider.previous()
        image = None
        if self.media_item:
            image = self.repository.retrieve(
                self.media_item, width=self.image_width, height=self.image_height
            )
        return self.media_item, image

    def next_media(self) -> Tuple[MediaItem, Image]:
        self.media_item = self.annotation_task_provider.next()
        image = None
        if self.media_item:
            image = self.repository.retrieve(self.media_item)

        return self.media_item, image

    def confirm(self, media_item):
        self.annotation_task_provider.confirm_current_task(media_item)

    def get_status(self) -> AnnotationTaskStatus:
        return self.annotation_task_provider.get_status()



from ipywidgets import HTML, Box, Label


class MetadataFormatter:
    @staticmethod
    def from_box(box: Box):
        metadata = {}
        if (
            hasattr(box, "description")
            and hasattr(box, "value")
            and not isinstance(box, Label)
            and not isinstance(box, HTML)
        ):
            key = box.description.lower().replace(" ", "_")
            metadata[key] = box.value
        else:
            if hasattr(box, "children"):
                for children_box in box.children:
                    metadata_children = MetadataFormatter.from_box(children_box)
                    metadata = {**metadata, **metadata_children}
        return metadata



import json
from datetime import datetime
from typing import List

from chosco.annotation.domain.aggregate_roots.media_item import MediaItem
from chosco.annotation.domain.annotation_task_provider import AnnotationTaskProvider
from chosco.annotation.domain.annotation_task_status import AnnotationTaskStatus


class JsonAnnotationTaskProvider(AnnotationTaskProvider):
    @staticmethod
    def from_filename(filename: str, pending_only: bool = False):
        with open(filename, "r") as infile:
            kdict = json.load(infile)
            done = kdict.get("done")
            done_media_items = (
                [MediaItem.from_dict(media_dict) for media_dict in done]
                if done
                else None
            )
            pending = kdict.get("pending")
            pending_media_items = (
                [MediaItem.from_dict(media_dict) for media_dict in pending]
                if pending
                else None
            )

            return JsonAnnotationTaskProvider(
                done_media_items,
                pending_media_items,
                kdict.get("session_id"),
                kdict.get("current_task_index"),
                filename,
                pending_only,
            )

    @staticmethod
    def create(tasks: List[MediaItem], session_id: str):
        done = list()
        pending = tasks
        return JsonAnnotationTaskProvider(done, pending, session_id)

    def __init__(
        self,
        done: List[MediaItem],
        pending: List[MediaItem],
        session_id: str,
        current_task_index: int = None,
        filename: str = None,
        pending_only: bool = False,
    ):
        self.done = done if isinstance(done, list) else list()
        self.pending = pending
        self.pending = pending if isinstance(pending, list) else list()
        self.total_tasks = self.done + self.pending
        self.current_task_index = current_task_index
        self.pending_only = pending_only
        if self.pending_only:
            self.current_task_index = len(self.done)
        self.session_id = session_id
        self.updated_at = None
        if self.pending is None or len(self.pending) < 1:
            raise AttributeError(
                f"Pending documents must have at least a task. {self.to_dict()}"
            )
        self.current_task = None
        self.filename = filename

    def to_dict(self):
        return {
            "updated_at": self.updated_at,
            "number_of_annotations": self.get_number_of_annotations(),
            "number_pending_annotations": self.get_number_pending_annotations(),
            "done": [field_item.to_dict() for field_item in self.done],
            "pending": [field_item.to_dict() for field_item in self.pending],
            "session_id": self.session_id,
            "current_task_index": self.current_task_index,
        }

    def __repr__(self):
        return json.dumps(self.to_dict(), indent=4)

    def __eq__(self, other):
        return self.to_dict() == other.to_dict()

    def save(self, filename):
        self.filename = filename
        with open(filename, "w") as outfile:
            json.dump(self.to_dict(), outfile, indent=4)

    def save_snapshot(self):
        if self.filename:
            self.save(self.filename)

    def get_current_index(self):
        return self.current_task_index

    def get_number_of_annotations(self):
        return len(self.done) if self.done else 0

    def get_number_pending_annotations(self):
        return len(self.pending) if self.pending else 0

    def get_total_annotations(self):
        return len(self.total_tasks)

    def get_status(self):
        return AnnotationTaskStatus(
            index=self.get_current_index(),
            number_of_annotations=self.get_number_of_annotations(),
            total_annotations=self.get_total_annotations(),
            current_task=self.current_task,
        )

    def next(self) -> MediaItem:
        self.current_task_index = (
            -1 if self.current_task_index is None else self.current_task_index
        )
        try:
            if self.current_task_index >= len(self.total_tasks) - 1:
                self.current_task = None
            else:
                self.current_task_index += 1
                self.current_task = self.total_tasks[self.current_task_index]
        except IndexError:
            self.current_task = None
        return self.current_task

    def previous(self) -> MediaItem:
        if self.current_task_index is None or self.current_task_index < 1:
            self.current_task = None
        else:
            self.current_task_index -= 1
            self.current_task = self.total_tasks[self.current_task_index]

        return self.current_task

    def confirm_current_task(self, task: MediaItem):
        if len(self.pending) > 0:
            try:
                self.pending.remove(task)
                self.done.append(task)
                self.updated_at = datetime.utcnow().strftime("%B %d %Y - %H:%M:%S")
                self.save_snapshot()
            except ValueError:
                pass



import os

from setuptools import find_packages, setup

CURRENT_DIR = os.path.abspath(os.path.dirname(__file__))
PACKAGE_NAME = "chosco"
VERSION = open("chosco/VERSION", "r").read()

# The text of the README file
with open(os.path.join(CURRENT_DIR, "README.md")) as fid:
    README = fid.read()

with open("requirements/requirements.txt") as f:
    required = f.read().splitlines()

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    description="Package on top of ipywidgets to label media resources on Notebook pages.",
    long_description=README,
    long_description_content_type="text/markdown",
    keywords=["notebook", "ipywidgets", "labeler"],
    url="https://github.com/alice-biometrics/chosco",
    author="ALiCE Biometrics",
    author_email="support@alicebiometrics.com",
    license="MIT",
    install_requires=required,
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
    ],
    packages=find_packages(exclude=("tests*",)),
    include_package_data=True,
    zip_safe=False,
)



import base64
import os
from typing import Union

from google.cloud import storage
from google.cloud.exceptions import NotFound
from google.cloud.storage import Blob
from IPython.core.display import Image, Video

from chosco.annotation.domain.aggregate_roots.media_item import MediaItem
from chosco.media.repository.domain.media_repository import MediaRepository


class GoogleCloudMultimediaRepository(MediaRepository):
    def __init__(self):
        self.storage_client = storage.Client()

    def retrieve(
        self, media_item: MediaItem, width: int = None, height: int = None
    ) -> Union[Image, Video]:
        try:
            blob = self._get_blob(media_item.media_id, media_item.media_bucket)
            data = blob.download_as_string(client=self.storage_client)
        except NotFound:
            raise Exception(
                f"Media {media_item.media_id} not found in Google Cloud repository"
            )
        extension = os.path.splitext(media_item.media_id)[-1].lower().replace(".", "")
        if extension in ["mp4", "avi", "mov"]:
            if extension == "mov":
                extension = "ogg"  # "quicktime" It seems to work only with ogg mime
            elif extension == "avi":
                extension = "x-msvideo"
            return Video(
                data=base64.b64encode(data).decode(),
                embed=True,
                width=width,
                height=height,
                mimetype="video/{}".format(extension),
                html_attributes="controls autoplay=autoplay",
            )
        else:
            return Image(
                data=data, format="jpg", unconfined=False, width=width, height=height
            )

    def _get_blob(self, media_id: str, bucket_name: str) -> Blob:
        bucket = self.storage_client.get_bucket(bucket_name)
        blob = bucket.blob(media_id)
        return blob
